from django.shortcuts import render

from rest_framework_simplejwt.views import TokenObtainPairView
from .serializers import CustomTokenObtainPairSerializer

class CustomTokenObtainPairView(TokenObtainPairView):
    """
    View that uses the custom serializer to return a JWT with extra claims.
    """
    serializer_class = CustomTokenObtainPairSerializer


from django.shortcuts import render

from rest_framework_simplejwt.views import TokenObtainPairView
from .serializers import CustomTokenObtainPairSerializer

class CustomTokenObtainPairView(TokenObtainPairView):
    """
    View that uses the custom serializer to return a JWT with extra claims.
    """
    serializer_class = CustomTokenObtainPairSerializer


from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from .models import Identity
from .serializers import IdentitySerializer
from django.db.models import Q

# class IdentityView(APIView):
#     """
#     Read-only API for Identity. Returns the single Identity instance.
#     Supports filtering by `service_type`.
#     """

#     def get(self, request):
#         service_type = request.query_params.get("service_type")  # Get filter parameter

#         if service_type:
#             identity = Identity.objects.filter(service_type=service_type).first()
#         else:
#             identity = Identity.objects.first()  # Get any Identity instance if no filter is applied

#         if identity:
#             serializer = IdentitySerializer(identity)
#             return Response(serializer.data)
        
#         return Response({"error": "Identity not found"}, status=status.HTTP_404_NOT_FOUND)
class IdentityView(APIView):
    """
    Read-only API for Identity. Returns the single Identity instance.
    Supports dynamic filtering based on query parameters.
    """

    def get(self, request):
        query_filters = Q()

        for key, value in request.query_params.items():
            if key in [field.name for field in Identity._meta.fields]:  # Ensure valid field
                field = Identity._meta.get_field(key)
                if field.is_relation:
                    related_model = field.related_model
                    if hasattr(related_model, "name"):  # Assume the related model has a 'name' field
                        key = f"{key}__name"

                query_filters &= Q(**{key: value})
        
        identity = Identity.objects.filter(query_filters).first()

        if identity:
            serializer = IdentitySerializer(identity)
            return Response(serializer.data)

        return Response({"error": "Identity not found"}, status=status.HTTP_404_NOT_FOUND)




from django.core.management.base import BaseCommand
from houseteli_administrator.models import ServiceType, Identity
from django.conf import settings


class Command(BaseCommand):
    help = 'Create default ServiceType and Identity instance'

    def handle(self, *args, **kwargs):
        # Create the default 'auth' ServiceType if it doesn't exist
        service_type, created = ServiceType.objects.get_or_create(
            name='auth',
            defaults={'description': 'Authentication Service'}
        )
        
        if created:
            self.stdout.write(self.style.SUCCESS("Created default 'auth' ServiceType"))
        else:
            self.stdout.write(self.style.SUCCESS("'auth' ServiceType already exists"))
        
        # Check if the Identity instance exists, if not, create it
        if not Identity.objects.exists():
            identity_name = getattr(settings, 'SERVICE_NAME', 'anon')
            identity_description = f"{identity_name} Service"  # Custom description logic

            identity = Identity.objects.create(
                name=identity_name,
                description=identity_description,
                enable_admin=True,
                service_type=service_type
            )

            self.stdout.write(self.style.SUCCESS(f"Created Identity: {identity.name}"))
        else:
            self.stdout.write(self.style.SUCCESS("Identity instance already exists"))



from django.contrib.auth.backends import ModelBackend, BaseBackend
from django.contrib.auth.models import AnonymousUser
import uuid
from django.conf import settings
import logging
logger = logging.getLogger(__name__)

class SessionEnhancingAuthBackend(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        """
        Override the authenticate method to add custom attributes to the session.
        """
        user = super().authenticate(request, username=username, password=password, **kwargs)

        if user and user.is_authenticated:
            # Set custom session attributes for authenticated user
            # extra_fields = ['is_active', 'is_superuser', 'is_staff', 'email', 'user_type']
            extra_fields = ['is_active', 'is_superuser', 'is_staff', 'email']
            existing_fields = [field for field in extra_fields if hasattr(user, field)]

            # map(lambda field: request.session.__setitem__(f"user_{field}", getattr(user, field)), existing_fields) # used map instead of for loop so I can practice using maps. But it does not actually set the items
            [request.session.__setitem__(f"user_{field}", getattr(user, field)) for field in existing_fields]

            request.session["account_type"] = settings.AUTH_IDENTIFIER
        
        return None



from dj_rest_auth.registration.serializers import RegisterSerializer
from rest_framework import serializers
from django.contrib.auth import get_user_model

User = get_user_model()

class CustomRegisterSerializer(RegisterSerializer):
    username = None
    email = serializers.EmailField(required=True)
    def custom_signup(self, request, user):
        user.email = self.data.get('email')
        user.save()


from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
# from django.conf import settings
# from decouple import config
# account_type = config('ACCOUNTTYPE', default=None)
class CustomTokenObtainPairSerializer(TokenObtainPairSerializer):
    """
    Custom serializer to add extra claims to the JWT.
    """
    @classmethod
    def get_token(cls, user):
        token = super().get_token(user)
        
        # Add custom claims
        token['email'] = user.email
        token['first_name'] = user.first_name
        token['last_name'] = user.last_name
        token['is_superuser'] = user.is_superuser
        # token['user_type'] = user.user_type
        return token
